/*
	
	GRIM DAWN
	
	For more information, visit us at http://www.grimdawn.com
	
*/

gd.quests.flavorEvent = {}

--
-- Spawns an NPC that can go hostile when talked to.
--

local npcId = 0
local npcSpawned = false

local NPCState = { Default = 0, Hostile = 1, Dead = 2 }

local npcStateObjects = orderedTable();
	npcStateObjects["NPC_DEAD"] = { state = NPCState.Dead, dbr = nil }
	npcStateObjects["NPC_HOSTILE"] = { state = NPCState.Hostile, dbr = "records/creatures/enemies/boss&quest/scriptednpc_hostile.dbr" }
	npcStateObjects[""	                    ] = { state = NPCState.Default,  dbr = "records/creatures/npcs/flavornpcs/npc_scriptednpc_01.dbr"}

local function CharacterCreateArgs()
	
	-- controls what level the NPC is when it switches to hostile, based on difficulty
	local maxLevel = 0

	if (Game.GetGameDifficulty() == Game.Difficulty.Normal) then
		maxLevel = 25
	elseif (Game.GetGameDifficulty() == Game.Difficulty.Epic) then
		maxLevel = 60
	else
		maxLevel = 80
	end
	
	if (Game.GetAveragePlayerLevel() < maxLevel) then
		maxLevel = 0
	end
	
	return maxLevel, nil
	
end
	
-- when the script entity loads, it determines what should appear based on the host's token state
function gd.quests.flavorEvent.npcOnAddToWorld(objectId)

	if Server then
	
		if (not npcSpawned) then

			npcId = objectId

			-- Create peaceful or active NPC
			local userdata = {}
			local didWork, newState = TokenStateBasedObjectSwap(objectId, userdata, npcStateObjects, Character, CharacterCreateArgs)
			
			Shared.setUserdata(objectId, userdata)

			-- Don't respawn NPC next time if spawned as hostile, he will be saved into the world
			if (newState == NPCState.Hostile) then
				npcSpawned = true
			end
			
		end
		
	end
	
end

gd.quests.flavorEvent.npcOnDestroy = ClearObjectUserdata


function gd.quests.flavorEvent.npcKilled()

	local player = Game.GetLocalPlayer()
	
	-- Gives the token state to all players present who have him set to hostile
	if (player != nil && player:HasToken("NPC_HOSTILE")) then
		player:GiveToken("NPC_DEAD")
		player:RemoveToken("NPC_HOSTILE")

	end

end


--Triggered by conversation hostile swap.

-- Gives the token state to all players present who have not already killed him 
function gd.quests.flavorEvent.hostileSwitchGlobalMP()

	local player = Game.GetLocalPlayer()
	
	if (player != nil && player:HasToken("NPC_DEAD") == false) then
		player:GiveToken("NPC_HOSTILE")
		
	end

end

function gd.quests.flavorEvent.hostileSwitch()

	if Server then
		if not npcSpawned then
			npcSpawned = true

			-- Update NPC to Hostile
			GiveTokenToLocalPlayer("NPC_HOSTILE")
			
			local didWork, newState = UpdateObjectSwap(npcId, npcStateObjects, Character, CharacterCreateArgs)
			
			-- Running this executes the function assigned to this call in questevents.lua for all players
			QuestGlobalEvent("npcHostileSwapToken")
		
		end
	
	end
	
end



--
-- Door Control for the second quest
-- the door opens when the player enters the nearby trigger volume after completing the quest.
-- the door is also opened automatically when the quest is first completed
--

local doorId = 0
local doorOpen = false

function gd.quests.flavorEvent.doorOnAddToWorld(objectId)

	if Server then
	
		doorId = objectId
		
	end

end

-- Called when trigger entered
function gd.quests.flavorEvent.openDoorTrigger(triggererId)

	if not doorOpen then
	
		local player = Player.Get(triggererId)
		
		-- Open door if player or the server has the token
		if (player != nil && (player:HasToken("TEST_DOOR") || player:ServerHasToken("TEST_DOOR"))) then
			QuestEvent("openTestDoor")
		end
		
	end

end

function gd.quests.flavorEvent.openDoorQuest()

	if Server then
		if not doorOpen then
			local door = Door.Get(doorId)
			door:Open()
			
			doorOpen = true
		end
		
	end

end


